@@ -290,6 +290,14 @@ public:
 
     void logMappedRanges();     //!< Log the mapped memory ranges to the default message logger.
 
+    /*! Memory accessor cacheing
+     * 
+     *  Memory accessor uses caching to reduce the number of calls / callbacks for memory images
+     *  This allows controlling / disabling of cacheing mechanisms.
+     *  Error returned if cache limits exceeded (4096 byte page, 256 pages)
+     */
+    ocsd_err_t setMemAccCacheing(const bool enable, const uint16_t page_size, const int nr_pages);
+
 /** @}*/
 
 /** @name Memory Accessors
@@ -413,6 +421,12 @@ private:
         const ocsd_mem_space_acc_t mem_space, void *p_cb_func, bool IDfn, const void *p_context);
     TrcPktProcI *getPktProcI(const uint8_t CSID);
 
+    // keep internal list of memory accessors created by this object.
+    void addMemAccessorToList(TrcMemAccessorBase* p_accessor);
+
+    // destroy mem accessors in use by this object
+    void destroyMemAccessors();
+
     ocsd_dcd_tree_src_t m_dcd_tree_type;
 
     IInstrDecode *m_i_instr_decode;
@@ -444,6 +458,9 @@ private:
 
     /**! demux stats block */
     ocsd_demux_stats_t m_demux_stats;
+
+    /**! List of accessors created by the decode tree */
+    std::list<TrcMemAccessorBase*> m_mem_accessors;
 };
 
 /** @}*/
