@@ -43,6 +43,8 @@
 #include "common/ocsd_gen_elem_stack.h"
 #include "opencsd/etmv4/trc_etmv4_stack_elem.h"
 
+#define OCSD_ENV_INSTR_RANGE_LIMIT "OPENCSD_INSTR_RANGE_LIMIT"
+
 class TrcStackElem;
 class TrcStackElemParam;
 class TrcStackElemCtxt;
@@ -109,8 +111,11 @@ protected:
     // sequencing error on packet processing - optionally continue
     ocsd_err_t handlePacketSeqErr(ocsd_err_t err, ocsd_trc_index_t index, const char *reason);
 
+    // inconsistent image for decode - optionally reset and continue
+    ocsd_err_t handleBadImageError(ocsd_trc_index_t index, const char* reason);
+
     // common packet error routine
-    ocsd_err_t handlePacketErr(ocsd_err_t err, ocsd_err_severity_t sev, ocsd_trc_index_t index, const char *reason);
+    ocsd_err_t handlePacketErr(ocsd_err_t err, ocsd_err_severity_t sev, ocsd_trc_index_t index, const char *reason, const unsync_info_t unsync_reason);
 
     ocsd_err_t addElemCC(TrcStackElemParam *pParamElem);
     ocsd_err_t addElemTS(TrcStackElemParam *pParamElem, bool withCC);
@@ -130,6 +135,12 @@ private:
         WP_NACC
     } WP_res_t;
 
+    /* Optional run limit for decoded instruction range. 
+     * Throw error if limit exceeded. Set by env var - use for debugging decode runs 
+     * which may be running away due to bad data
+     */
+    int m_num_instr_range_limit;
+
     typedef struct {
         ocsd_vaddr_t st_addr;
         ocsd_vaddr_t en_addr;
@@ -243,6 +254,34 @@ private:
 
     TrcAddrReturnStack m_return_stack;  //!< the address return stack.
 
+    // range address check to look for possible incorrect input code memory images.
+    struct {
+        ocsd_vaddr_t next_st_addr;  // expected address for next range
+        bool valid;
+    } m_next_range_check;
+
+    void nextRangeCheckClear() {
+        m_next_range_check.valid = false;
+    };
+
+    void nextRangeCheckSet(const ocsd_vaddr_t addr) {
+        m_next_range_check.valid = true;
+        m_next_range_check.next_st_addr = addr;
+    };
+
+    bool nextRangeCheckOK(const ocsd_vaddr_t addr) {
+        if (m_next_range_check.valid) {
+            return (bool)(m_next_range_check.next_st_addr == addr);
+        }
+        // no check info - just return OK
+        return true;
+    };
+
+    // consistency check flags
+    bool m_direct_br_chk;
+    bool m_strict_br_chk;
+    bool m_range_cont_chk;
+
 //** output element handling
     OcsdGenElemStack m_out_elem;  //!< output element stack.
     OcsdTraceElement &outElem() { return m_out_elem.getCurrElem(); };   //!< current  out element
