@@ -2,7 +2,7 @@
  * \file       trc_pkt_types_etmv4.h
  * \brief      OpenCSD : ETMv4 / ETE packet info
  * 
- * \copyright  Copyright (c) 2015,2019 ARM Limited. All Rights Reserved.
+ * \copyright  Copyright (c) 2015,2019,2022 ARM Limited. All Rights Reserved.
  */
 
 
@@ -73,7 +73,7 @@ typedef enum _ocsd_etmv4_i_pkt_type
         ETM4_PKT_I_EXCEPT_RTN =         0x07,   /*!< b00000111 (ETE invalid) */
 
         /* unused encoding              0x08         b00001000 */
-        ETE_PKT_I_COMMIT_WIN_MV =       0x09,   /*!  b00001001 (ETE only - unused in current versions) */
+        ETE_PKT_I_ITE =                 0x09,   /*!  b00001001 (ETE only) */
         ETE_PKT_I_TRANS_ST =            0x0A,   /*!  b00001010 (ETE only) */
         ETE_PKT_I_TRANS_COMMIT =        0x0B,   /*!  b00001011 (ETE only) */
 
@@ -274,6 +274,11 @@ typedef struct _ocsd_etmv4_i_pkt
         };
     } Q_pkt;
 
+    struct {
+        uint8_t el;
+        uint64_t value;
+    } ite_pkt;
+
     //! valid bits for packet elements (addresses have their own valid bits).
     union {
         uint32_t val;
@@ -382,7 +387,10 @@ typedef struct _ocsd_etmv4_cfg
 
 #define ETE_ARCH_VERSION 0x5
 
-#define ETE_OPFLG_PKTDEC_SRCADDR_N_ATOMS 0x00010000 /**< Split source address output ranges for N-atoms */
+#define ETE_OPFLG_PKTDEC_SRCADDR_N_ATOMS    0x00010000 /**< Split source address output ranges for N-atoms */
+#define ETM4_OPFLG_PKTDEC_AA64_OPCODE_CHK   0x00020000 /**< check for invalid AA64 opcodes. (MSW == 0x0000) */
+
+#define ETE_ETM4_OPFLG_MASK (ETE_OPFLG_PKTDEC_SRCADDR_N_ATOMS | ETM4_OPFLG_PKTDEC_AA64_OPCODE_CHK)
 
 /** @}*/
 /** @}*/
