@@ -39,8 +39,9 @@
 
 #define DCD_NAME "DCD_ETMV4"
 
-static const uint32_t ETMV4_SUPPORTED_DECODE_OP_FLAGS = OCSD_OPFLG_PKTDEC_COMMON |
-                        ETE_OPFLG_PKTDEC_SRCADDR_N_ATOMS;
+static const uint32_t ETMV4_SUPPORTED_DECODE_OP_FLAGS =
+    OCSD_OPFLG_PKTDEC_COMMON |  /* common op flags */
+    ETE_ETM4_OPFLG_MASK;        /* ete - etm4 op flags */
 
 TrcPktDecodeEtmV4I::TrcPktDecodeEtmV4I()
     : TrcPktDecodeBase(DCD_NAME)
@@ -118,13 +119,18 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::processPacket()
             err = decodePacket();
             if (err)
             {
-#ifdef OCSD_WARN_UNSUPPORTED
-                if (err == OCSD_ERR_UNSUPP_DECODE_PKT)
-                    resp = OCSD_RESP_WARN_CONT;
+                // may want to continue through bad packets
+                if ((err == OCSD_ERR_BAD_DECODE_PKT) || (err == OCSD_ERR_UNSUPP_DECODE_PKT))
+                {
+                    if (getComponentOpMode() & OCSD_OPFLG_PKTDEC_HALT_BAD_PKTS)
+                        resp = OCSD_RESP_FATAL_INVALID_DATA;
+                    else if (getComponentOpMode() & OCSD_OPFLG_PKTDEC_ERROR_BAD_PKTS)
+                        resp = OCSD_RESP_ERR_CONT;
+                    else
+                        resp = OCSD_RESP_WARN_CONT;
+                }
                 else
-#else
-                resp = OCSD_RESP_FATAL_INVALID_DATA;
-#endif
+                    resp = OCSD_RESP_FATAL_INVALID_DATA;
 
                 bPktDone = true;
             }
@@ -227,6 +233,12 @@ ocsd_err_t TrcPktDecodeEtmV4I::onProtocolConfig()
         err = OCSD_ERR_HW_CFG_UNSUPP;
         LogError(ocsdError(OCSD_ERR_SEV_ERROR,OCSD_ERR_HW_CFG_UNSUPP,"ETMv4 instruction decode : Trace on conditional non-branch elements not supported."));
     }
+
+    // set consistency check flags 
+    m_direct_br_chk = (bool)(getComponentOpMode() & OCSD_OPFLG_N_UNCOND_DIR_BR_CHK);
+    m_strict_br_chk = (bool)(getComponentOpMode() & OCSD_OPFLG_STRICT_N_UNCOND_BR_CHK);
+    m_range_cont_chk = (bool)(getComponentOpMode() & OCSD_OPFLG_CHK_RANGE_CONTINUE);
+    
     return err;
 }
 
@@ -241,6 +253,20 @@ void TrcPktDecodeEtmV4I::initDecoder()
     m_CSID = 0;
     m_IASize64 = false;
 
+    // set debug range limit - look for debugging env var
+    char* env_var;
+    long env_val;
+
+    m_num_instr_range_limit = 0;
+    if ((env_var = getenv(OCSD_ENV_INSTR_RANGE_LIMIT)) != NULL)
+    {
+        env_val = strtol(env_var, NULL, 0);
+        /* if valid number set limit */
+        if (env_val > 0)
+            m_num_instr_range_limit = env_val;
+
+    }
+
     // elements associated with data trace
 #ifdef DATA_TRACE_SUPPORTED
     m_p0_key_max = 0;
@@ -271,6 +297,7 @@ void TrcPktDecodeEtmV4I::resetDecoder()
     m_last_IS = 0;
     clearElemRes();
     m_ete_first_ts_marker = false;
+    nextRangeCheckClear();
 
     // elements associated with data trace
 #ifdef DATA_TRACE_SUPPORTED
@@ -590,20 +617,7 @@ ocsd_err_t TrcPktDecodeEtmV4I::decodePacket()
     case ETM4_PKT_I_NUM_DS_MKR:
     case ETM4_PKT_I_UNNUM_DS_MKR:
         // all currently unsupported
-        {
-        ocsd_err_severity_t sev = OCSD_ERR_SEV_ERROR;
-#ifdef OCSD_WARN_UNSUPPORTED
-        sev = OCSD_ERR_SEV_WARN;
-        //resp = OCSD_RESP_WARN_CONT;
-#else
-        //resp = OCSD_RESP_FATAL_INVALID_DATA;
-#endif
-        err = OCSD_ERR_UNSUPP_DECODE_PKT;
-        if (sev == OCSD_ERR_SEV_WARN)
-                        LogError(ocsdError(sev, err, "Data trace related, unsupported packet type."));
-        else 
-            err = handlePacketSeqErr(err, m_index_curr_pkt, "Data trace related, unsupported packet type.");
-        }
+        err = handlePacketSeqErr(OCSD_ERR_UNSUPP_DECODE_PKT, m_index_curr_pkt, "Data trace related, unsupported packet type.");
         break;
 
     default:
@@ -684,8 +698,13 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::resolveElements()
                     err = discardElements();
             }
 
-            if (err != OCSD_OK)
-                resp = OCSD_RESP_FATAL_INVALID_DATA;
+            if (err != OCSD_OK) {
+                // has the error reset the decoder?
+                if (m_curr_state == NO_SYNC)
+                    resp = OCSD_RESP_ERR_CONT;
+                else
+                    resp = OCSD_RESP_FATAL_INVALID_DATA;
+            }
         }
         
         // break out on error or wait request.
@@ -732,6 +751,7 @@ ocsd_err_t TrcPktDecodeEtmV4I::commitElements()
             {
                 // indicates a trace restart - beginning of trace or discontinuiuty
             case P0_TRC_ON:
+                nextRangeCheckClear();
                 err = m_out_elem.addElemType(pElem->getRootIndex(), OCSD_GEN_TRC_ELEM_TRACE_ON);
                 if (!err)
                 {
@@ -800,17 +820,20 @@ ocsd_err_t TrcPktDecodeEtmV4I::commitElements()
                     {
                         ocsd_atm_val atom = pAtomElem->commitOldest();
 
-                        // check if prev atom left us an indirect address target on the return stack
+                        // check if prev atom was indirect branch - may need address from return stack
                         if ((err = returnStackPop()) != OCSD_OK)
                             break;
 
                         // if address and context do instruction trace follower.
                         // otherwise skip atom and reduce committed elements
+                        // allow for insufficient program image.
                         if (!m_need_ctxt && !m_need_addr)
                         {
-                            err = processAtom(atom);
+                            if ((err = processAtom(atom)) != OCSD_OK)
+                                break;
                         }
-                        m_elem_res.P0_commit--; // mark committed 
+                        if (m_elem_res.P0_commit)
+                            m_elem_res.P0_commit--; // mark committed 
                     }
                     if (!pAtomElem->isEmpty())
                         bPopElem = false;   // don't remove if still atoms to process.
@@ -823,11 +846,13 @@ ocsd_err_t TrcPktDecodeEtmV4I::commitElements()
                 if ((err = returnStackPop()) != OCSD_OK)
                     break;
 
+                nextRangeCheckClear();
                 err = processException();  // output trace + exception elements.
                 m_elem_res.P0_commit--;
                 break;
 
             case P0_EXCEP_RET:
+                nextRangeCheckClear();
                 err = m_out_elem.addElemType(pElem->getRootIndex(), OCSD_GEN_TRC_ELEM_EXCEPTION_RET);
                 if (!err)
                 {
@@ -844,11 +869,13 @@ ocsd_err_t TrcPktDecodeEtmV4I::commitElements()
                 break;
 
             case P0_SRC_ADDR:
+                nextRangeCheckClear();
                 err = processSourceAddress();
                 m_elem_res.P0_commit--;
                 break;
 
             case P0_Q:
+                nextRangeCheckClear();
                 err = processQElement();
                 m_elem_res.P0_commit--;
 				break;
@@ -859,6 +886,7 @@ ocsd_err_t TrcPktDecodeEtmV4I::commitElements()
             case P0_TRANS_COMMIT:
             case P0_TRANS_FAIL:
             case P0_TRANS_TRACE_INIT:
+                nextRangeCheckClear();
                 err = processTransElem(pElem);
                 break;
 
@@ -1338,6 +1366,20 @@ ocsd_err_t TrcPktDecodeEtmV4I::processAtom(const ocsd_atm_val atom)
                     m_return_stack.push(nextAddr, m_instr_info.isa);
 
             }
+            else if (m_direct_br_chk || m_strict_br_chk)  // consistency checks on N atoms?
+            {
+                // N atom - but direct branch instruction not conditional - bad input image?
+                if (!m_instr_info.is_conditional)
+                {
+                    // Some ETM IP incorrectly trace a taken branch to next instruction as N
+                    // look for branch where it is not next instruction if direct branch checks only
+                    if (((m_instr_info.branch_addr != nextAddr) && m_direct_br_chk) || m_strict_br_chk)
+                    {
+                        err = handleBadImageError(pElem->getRootIndex(), "Bad program image - N Atom on unconditional direct BR.\n");
+                        return err;
+                    }
+                }
+            }
             break;
 
         case OCSD_INSTR_BR_INDIRECT:
@@ -1346,17 +1388,45 @@ ocsd_err_t TrcPktDecodeEtmV4I::processAtom(const ocsd_atm_val atom)
                 m_need_addr = true; // indirect branch taken - need new address.
                 if (m_instr_info.is_link)
                     m_return_stack.push(nextAddr,m_instr_info.isa);
-                m_return_stack.set_pop_pending();  // need to know next packet before we know what is to happen
+
+                // mark last atom as BR indirect - if no address next need addr from return stack.
+                m_return_stack.set_pop_pending();  
 
                 /* ETE does not have ERET trace packets - however to maintain the illusion if we see an ERET
                    output a gen elem ERET packet */
                 if (isETEConfig() && (m_instr_info.sub_type == OCSD_S_INSTR_V8_ERET))
                     ETE_ERET = true;
             }
+            else if (m_strict_br_chk) // consistency checks on N atoms?
+            {
+                // N atom - check if conditional - only in strict check mode.
+                if (!m_instr_info.is_conditional)
+                {
+                    err = handleBadImageError(pElem->getRootIndex(), "Bad program image - N Atom on unconditional indirect BR.\n");
+                    return err;
+                }
+            }
             break;
         }
         setElemTraceRange(outElem(), addr_range, (atom == ATOM_E), pElem->getRootIndex());
 
+        // check for discontinuity in address ranges where incorrect memory images supplied to decoder.
+        if (m_range_cont_chk)
+        {
+            // do the previous range chack.
+            if (!nextRangeCheckOK(addr_range.st_addr))
+            {
+                err = handleBadImageError(pElem->getRootIndex(), "Discontinuous ranges - Inconsistent program image for decode\n");
+                return err;
+            }
+
+            if (atom == ATOM_N)
+                // branch not taken - expect next range to be continuous
+                nextRangeCheckSet(nextAddr);
+            else
+                nextRangeCheckClear();  // branch taken - not continuous
+        }
+
         if (ETE_ERET)
         {
             err = m_out_elem.addElemType(pElem->getRootIndex(), OCSD_GEN_TRC_ELEM_EXCEPTION_RET);
@@ -1368,6 +1438,7 @@ ocsd_err_t TrcPktDecodeEtmV4I::processAtom(const ocsd_atm_val atom)
     {
         // no waypoint - likely inaccessible memory range.
         m_need_addr = true; // need an address update 
+        nextRangeCheckClear();
 
         if(addr_range.st_addr != addr_range.en_addr)
         {
@@ -1383,12 +1454,15 @@ ocsd_err_t TrcPktDecodeEtmV4I::processAtom(const ocsd_atm_val atom)
         {
             outElem().setType(OCSD_GEN_TRC_ELEM_ADDR_NACC);
             outElem().st_addr = m_instr_info.instr_addr;
+            outElem().exception_number = (uint32_t)getCurrMemSpace();
         }
     }
     return err;
 }
 
 // Exception processor
+#define M_CLASS_TAIL_ADDR 0xFFFFFFFE
+
 ocsd_err_t TrcPktDecodeEtmV4I::processException()
 {
     ocsd_err_t err;
@@ -1401,6 +1475,7 @@ ocsd_err_t TrcPktDecodeEtmV4I::processException()
     ocsd_trc_index_t excep_pkt_index;
     WP_res_t WPRes = WP_NOT_FOUND;
     bool ETE_resetPkt = false;
+    bool bMTailChain = false;
 
     // grab the exception element off the stack
     pExceptElem = dynamic_cast<TrcStackElemExcept *>(m_P0_stack.back());  // get the exception element
@@ -1472,8 +1547,12 @@ ocsd_err_t TrcPktDecodeEtmV4I::processException()
 
     if (!ETE_resetPkt)
     {
+        /* check for M class tail chain / deferred exceptions */        
+        if (m_config->coreProfile() == profile_CortexM)
+            bMTailChain = (excep_ret_addr == M_CLASS_TAIL_ADDR);
+
         // if the preferred return address is not the end of the last output range...
-        if (m_instr_info.instr_addr != excep_ret_addr)
+        if ((m_instr_info.instr_addr < excep_ret_addr) && !bMTailChain)
         {
             bool range_out = false;
             instr_range_t addr_range;
@@ -1529,6 +1608,7 @@ ocsd_err_t TrcPktDecodeEtmV4I::processException()
 
             outElem().setType(OCSD_GEN_TRC_ELEM_ADDR_NACC);
             outElem().st_addr = m_instr_info.instr_addr;
+            outElem().exception_number = (uint32_t)getCurrMemSpace();
 
             // used the element - need another for the final exception packet.
             if ((err = m_out_elem.addElem(excep_pkt_index)))
@@ -1541,7 +1621,13 @@ ocsd_err_t TrcPktDecodeEtmV4I::processException()
 
     // add end address as preferred return address to end addr in element
     outElem().en_addr = excep_ret_addr;
+    
     outElem().excep_ret_addr = 1;
+    if (bMTailChain) 
+    {
+        outElem().excep_ret_addr = 0;
+        outElem().excep_M_tail_chain = 1;
+    }
     outElem().excep_ret_addr_br_tgt = branch_target;
     outElem().exception_number = pExceptElem->getExcepNum();
 
@@ -1703,6 +1789,7 @@ ocsd_err_t TrcPktDecodeEtmV4I::processSourceAddress()
         // can't access - no bytes returned - output nacc.
         err = m_out_elem.addElemType(pElem->getRootIndex(), OCSD_GEN_TRC_ELEM_ADDR_NACC);
         outElem().setAddrStart(srcAddr.val);
+        outElem().exception_number = (uint32_t)getCurrMemSpace();
         return err;
     }
 
@@ -1803,6 +1890,7 @@ ocsd_err_t TrcPktDecodeEtmV4I::processSourceAddress()
                     if (err)
                         return err;
                     outElem().setAddrStart(srcAddr.val);
+                    outElem().exception_number = (uint32_t)getCurrMemSpace();
 
                     // force range to the one instruction
                     out_range.num_instr = 1;
@@ -1886,6 +1974,15 @@ ocsd_err_t TrcPktDecodeEtmV4I::traceInstrToWP(instr_range_t &range, WP_res_t &WP
             // not enough memory accessible.
             WPRes = WP_NACC;
         }
+
+        if (m_num_instr_range_limit)
+        {
+            if (range.num_instr > (uint32_t)m_num_instr_range_limit)
+            {
+                err = OCSD_ERR_I_RANGE_LIMIT_OVERRUN;
+                LogError(ocsdError(OCSD_ERR_SEV_ERROR, err, "Decode Instruction Range Limit Overrun"));
+            }
+        }
     }
     // update the range decoded address in the output packet.
     range.en_addr = m_instr_info.instr_addr;
@@ -1930,15 +2027,20 @@ ocsd_err_t TrcPktDecodeEtmV4I::handleBadPacket(const char *reason, ocsd_trc_inde
     if (getComponentOpMode() & OCSD_OPFLG_PKTDEC_ERROR_BAD_PKTS)
         sev = OCSD_ERR_SEV_ERROR;
 
-    return handlePacketErr(OCSD_ERR_BAD_DECODE_PKT, sev, index, reason);
+    return handlePacketErr(OCSD_ERR_BAD_DECODE_PKT, sev, index, reason, UNSYNC_BAD_PACKET);
 }
 
 ocsd_err_t TrcPktDecodeEtmV4I::handlePacketSeqErr(ocsd_err_t err, ocsd_trc_index_t index, const char *reason)
 {
-    return handlePacketErr(err, OCSD_ERR_SEV_ERROR, index, reason);
+    return handlePacketErr(err, OCSD_ERR_SEV_ERROR, index, reason, UNSYNC_BAD_PACKET);
 }
 
-ocsd_err_t TrcPktDecodeEtmV4I::handlePacketErr(ocsd_err_t err, ocsd_err_severity_t sev, ocsd_trc_index_t index, const char *reason)
+ocsd_err_t TrcPktDecodeEtmV4I::handleBadImageError(ocsd_trc_index_t index, const char* reason)
+{
+    return handlePacketErr(OCSD_ERR_BAD_DECODE_IMAGE, OCSD_ERR_SEV_ERROR, index, reason, UNSYNC_BAD_IMAGE);
+}
+
+ocsd_err_t TrcPktDecodeEtmV4I::handlePacketErr(ocsd_err_t err, ocsd_err_severity_t sev, ocsd_trc_index_t index, const char *reason, const unsync_info_t unsync_reason)
 {
     bool resetOnBadPackets = true;
 
@@ -1952,14 +2054,13 @@ ocsd_err_t TrcPktDecodeEtmV4I::handlePacketErr(ocsd_err_t err, ocsd_err_severity
         // switch to unsync - clear decode state
         resetDecoder();
         m_curr_state = NO_SYNC;
-        m_unsync_eot_info = UNSYNC_BAD_PACKET;
-        err = OCSD_OK;
+        m_unsync_eot_info = unsync_reason;
+        //err = OCSD_OK;
     }
     return err;
 
 }
 
-
 inline ocsd_mem_space_acc_t TrcPktDecodeEtmV4I::getCurrMemSpace()
 {
     static ocsd_mem_space_acc_t SMemSpace[] = {
@@ -1976,12 +2077,37 @@ inline ocsd_mem_space_acc_t TrcPktDecodeEtmV4I::getCurrMemSpace()
         OCSD_MEM_SPACE_EL3
     };
 
+    static ocsd_mem_space_acc_t RMemSpace[] = {
+        OCSD_MEM_SPACE_EL1R,
+        OCSD_MEM_SPACE_EL1R,
+        OCSD_MEM_SPACE_EL2R,
+        OCSD_MEM_SPACE_ROOT
+    };
+
     /* if no valid EL value - just use S/NS */
     if (!outElem().context.el_valid)
         return  m_is_secure ? OCSD_MEM_SPACE_S : OCSD_MEM_SPACE_N;
-    
+
     /* mem space according to EL + S/NS */
+    ocsd_mem_space_acc_t mem_space = OCSD_MEM_SPACE_NONE;
     int el = (int)(outElem().context.exception_level) & 0x3;
-    return m_is_secure ? SMemSpace[el] : NSMemSpace[el];
+    
+    switch (outElem().context.security_level)
+    {
+    case ocsd_sec_root:
+        mem_space = OCSD_MEM_SPACE_ROOT;
+        break;
+    case ocsd_sec_realm:
+        mem_space = RMemSpace[el];
+        break;
+    case ocsd_sec_nonsecure:
+        mem_space = NSMemSpace[el];
+        break;
+    case ocsd_sec_secure:
+        mem_space = SMemSpace[el];
+        break;
+    };
+    
+    return mem_space;
 }
 /* End of File trc_pkt_decode_etmv4i.cpp */
