@@ -275,6 +275,14 @@ void EtmV4ITrcPacket::toString(std::string &str) const
             }
         }
         break;
+
+    case ETE_PKT_I_ITE:
+        {
+            std::ostringstream oss;
+            oss << "; EL" << std::dec << (int)ite_pkt.el << "; Payload=0x" << std::hex << ite_pkt.value;
+            str += oss.str();
+        }
+        break;
     }
 
 }   
@@ -360,12 +368,7 @@ const char *EtmV4ITrcPacket::packetTypeName(const ocsd_etmv4_i_pkt_type type, co
         pName = "I_EXCEPT_RTN";
         pDesc = "Exception Return.";
         break;
-
-    case ETE_PKT_I_COMMIT_WIN_MV:
-        pName = "I_COMMIT_WIN_MV";
-        pDesc = "Commit window move.";
-        break;
-
+    
     case ETE_PKT_I_TRANS_ST:
         pName = "I_TRANS_ST";
         pDesc = "Transaction Start.";
@@ -494,7 +497,7 @@ const char *EtmV4ITrcPacket::packetTypeName(const ocsd_etmv4_i_pkt_type type, co
 
     case ETM4_PKT_I_ADDR_CTXT_L_32IS1:
         pName = "I_ADDR_CTXT_L_32IS1";
-        pDesc = "Address & Context, Long, 32 bit, IS0.";
+        pDesc = "Address & Context, Long, 32 bit, IS1.";
         break;
 
     case ETM4_PKT_I_ADDR_CTXT_L_64IS0:
@@ -642,6 +645,11 @@ const char *EtmV4ITrcPacket::packetTypeName(const ocsd_etmv4_i_pkt_type type, co
         pDesc = "Transaction Fail.";
         break;
 
+    case ETE_PKT_I_ITE:
+        pName = "I_ITE";
+        pDesc = "Instrumentation";
+        break;
+
     default:
         break;
     }
@@ -658,7 +666,11 @@ void EtmV4ITrcPacket::contextStr(std::string &ctxtStr) const
         std::ostringstream oss;
         if(context.updated)
         {           
-            oss << "Ctxt: " << (context.SF ? "AArch64," : "AArch32, ") << "EL" << context.EL << ", " << (context.NS ? "NS; " : "S; ");
+            oss << "Ctxt: " << (context.SF ? "AArch64," : "AArch32, ") << "EL" << context.EL << ", ";
+            if (context.NSE)
+                oss << (context.NS ? "Realm; " : "Root; ");
+            else
+                oss << (context.NS ? "NS; " : "S; ");
             if(context.updated_c)
             {
                 oss << "CID=0x" << std::hex << std::setfill('0') << std::setw(8) << context.ctxtID << "; ";
