@@ -51,13 +51,14 @@ DecodeTree *DecodeTree::CreateDecodeTree(const ocsd_dcd_tree_src_t src_type, uin
         if(dcd_tree->initialise(src_type, formatterCfgFlags))
         {
             s_trace_dcd_trees.push_back(dcd_tree);
+            s_instruction_decoder.envSetAA64_errOnBadOpcode();
         }
         else 
         {
             delete dcd_tree;
             dcd_tree = 0;
         }
-    }
+    }    
     return dcd_tree;
 }
 
@@ -112,6 +113,7 @@ DecodeTree::DecodeTree() :
 
 DecodeTree::~DecodeTree()
 {
+    destroyMemAccessors();
     destroyMemAccMapper();
     for(uint8_t i = 0; i < 0x80; i++)
     {
@@ -195,9 +197,17 @@ ocsd_err_t DecodeTree::createMemAccMapper(memacc_mapper_t type /* = MEMACC_MAP_G
     // set the access interface
     if(m_default_mapper)
     {
+        bool enableCaching;
+        int cachePageSize, cachePageNum;
+        
+
         m_created_mapper = true;
         setMemAccessI(m_default_mapper);
         m_default_mapper->setErrorLog(s_i_error_logger);
+        TrcMemAccCache::getenvMemaccCacheSizes(enableCaching, cachePageSize, cachePageNum);
+        if ((m_default_mapper->setCacheSizes(cachePageSize, cachePageNum) != OCSD_OK) ||
+            (m_default_mapper->enableCaching(enableCaching) != OCSD_OK))
+            destroyMemAccMapper();
     }
 
     return (m_default_mapper != 0) ? OCSD_OK : OCSD_ERR_MEM;
@@ -220,12 +230,50 @@ void DecodeTree::destroyMemAccMapper()
     }
 }
 
+void DecodeTree::addMemAccessorToList(TrcMemAccessorBase* p_accessor)
+{
+    m_mem_accessors.push_back(p_accessor);
+}
+
+// destroy mem accessors in use by this object
+void DecodeTree::destroyMemAccessors()
+{
+    std::list<TrcMemAccessorBase*>::iterator it;
+
+    it = m_mem_accessors.begin();
+    while (it != m_mem_accessors.end())
+    {
+        TrcMemAccFactory::DestroyAccessor(*it);
+        it++;
+    }
+    m_mem_accessors.clear();
+}
+
 void DecodeTree::logMappedRanges()
 {
     if(m_default_mapper)
         m_default_mapper->logMappedRanges();
 }
 
+ocsd_err_t DecodeTree::setMemAccCacheing(const bool enable, const uint16_t page_size, const int nr_pages)
+{
+    ocsd_err_t err = OCSD_OK;
+
+    if (!m_default_mapper)
+        return OCSD_ERR_NOT_INIT;
+
+    if (enable)
+    {
+        // set cache sizes - error if params out of limits
+        err = m_default_mapper->setCacheSizes(page_size, nr_pages, true);
+        if (err == OCSD_OK)
+            err = m_default_mapper->enableCaching(true);
+    }
+    else
+        err = m_default_mapper->enableCaching(false);
+    return err;
+}
+
 /* Memory accessor creation - all on default mem accessor using the 0 CSID for global core space. */
 ocsd_err_t DecodeTree::addBufferMemAcc(const ocsd_vaddr_t address, const ocsd_mem_space_acc_t mem_space, const uint8_t *p_mem_buffer, const uint32_t mem_length)
 {
@@ -249,8 +297,10 @@ ocsd_err_t DecodeTree::addBufferMemAcc(const ocsd_vaddr_t address, const ocsd_me
         else
             err = OCSD_ERR_MEM;    // wrong type of object - treat as mem error
 
-        if(err != OCSD_OK)
+        if (err != OCSD_OK)
             TrcMemAccFactory::DestroyAccessor(p_accessor);
+        else
+            addMemAccessorToList(p_accessor);
     }
     return err;
 }
@@ -277,8 +327,10 @@ ocsd_err_t DecodeTree::addBinFileMemAcc(const ocsd_vaddr_t address, const ocsd_m
         else
             err = OCSD_ERR_MEM;    // wrong type of object - treat as mem error
 
-        if(err != OCSD_OK)
+        if (err != OCSD_OK)
             TrcMemAccFactory::DestroyAccessor(p_accessor);
+        else
+            addMemAccessorToList(p_accessor);
     }
     return err;
 
@@ -319,8 +371,10 @@ ocsd_err_t DecodeTree::addBinFileRegionMemAcc(const ocsd_file_mem_region_t *regi
         else
             err = OCSD_ERR_MEM;    // wrong type of object - treat as mem error
 
-        if(err != OCSD_OK)
+        if (err != OCSD_OK)
             TrcMemAccFactory::DestroyAccessor(p_accessor);
+        else
+            addMemAccessorToList(p_accessor);
     }
     return err;
 }
@@ -379,8 +433,10 @@ ocsd_err_t DecodeTree::initCallbackMemAcc(const ocsd_vaddr_t st_address, const o
         else
             err = OCSD_ERR_MEM;    // wrong type of object - treat as mem error
 
-        if(err != OCSD_OK)
+        if (err != OCSD_OK)
             TrcMemAccFactory::DestroyAccessor(p_accessor);
+        else
+            addMemAccessorToList(p_accessor);
     }
     return err;
 }
@@ -416,6 +472,10 @@ ocsd_err_t DecodeTree::createDecoder(const std::string &decoderName, const int c
         crtFlags |= OCSD_CREATE_FLG_INST_ID;
     }
 
+    // check for the aa64 check.
+    if (createFlags & ETM4_OPFLG_PKTDEC_AA64_OPCODE_CHK)
+        s_instruction_decoder.setAA64_errOnBadOpcode(true);
+
     // create the decode element to attach to the channel.
     if((err = createDecodeElement(CSID)) != OCSD_OK)
         return err;
