@@ -40,6 +40,8 @@
 #include <iostream>
 #include <sstream>
 #include <cstring>
+#include <chrono>
+#include <ctime>
 
 #include "opencsd.h"              // the library
 #include "trace_snapshots.h"    // the snapshot reading test library
@@ -73,10 +75,12 @@ static int test_waits = 0;
 static bool dstream_format = false;
 static bool tpiu_format = false;
 static bool has_hsync = false;
-static bool src_addr_n = false;
 static bool stats = false;
 static bool profile = false;
 static bool multi_session = false;  // decode all buffers in snapshot as same config.
+static bool no_time_print = false; // dont print test eleapsed time (easier for regression comparisons)
+
+static uint32_t add_create_flags = 0;
 
 static bool macc_cache_disable = false;
 static uint32_t macc_cache_page_size = 0;
@@ -204,6 +208,13 @@ void print_help()
     oss << "-o_raw_unpacked     Output raw unpacked trace data per ID\n";
     oss << "-src_addr_n         ETE protocol: Split source address ranges on N atoms\n";
     oss << "-stats              Output packet processing statistics (if available).\n";
+    oss << "-no_time_print      Do not output the elapsed time for tests.\n";
+    oss << "\nConsistency checks\n\n";
+    oss << "-aa64_opcode_chk    Check for correct AA64 opcodes (MSW != 0x0000)\n";
+    oss << "-direct_br_cond     Check for incorrect N atom on direct unconditional branches\n";
+    oss << "-strict_br_cond     Strict conditional checks - look for incorrect N atom on all unconditional branches.\n";
+    oss << "-range_cont         Range continuity checks - check next range after N atom is continuous.\n";
+    oss << "-halt_err           Halt on bad packet error (default attempts to resync).\n";
     oss << "\nDevelopment:\nOptions used during develop and test of the library\n\n";
     oss << "-profile            Mute logging output while profiling library performance\n";
     oss << "-test_waits <N>     Force wait from packet printer for N packets - test the wait/flush mechanisms for the decoder\n";
@@ -416,7 +427,7 @@ bool process_cmd_line_opts(int argc, char* argv[])
             }
             else if (strcmp(argv[optIdx], "-src_addr_n") == 0)
             {
-                src_addr_n = true;
+                add_create_flags |= ETE_OPFLG_PKTDEC_SRCADDR_N_ATOMS;
             }
             else if (strcmp(argv[optIdx], "-stats") == 0)
             {
@@ -427,6 +438,10 @@ bool process_cmd_line_opts(int argc, char* argv[])
                 print_help();
                 bOptsOK = false;
             }
+            else if (strcmp(argv[optIdx],"-no_time_print") == 0)
+            {
+                no_time_print = true;
+            }
             else if((opt == "-logstdout") || (opt == "-logstderr") || 
                 (opt == "-logfile") || (opt == "-logfilename"))
             {
@@ -456,6 +471,26 @@ bool process_cmd_line_opts(int argc, char* argv[])
             {
                 profile = true;
             }
+            else if (strcmp(argv[optIdx], "-direct_br_cond") == 0)
+            {
+                add_create_flags |= OCSD_OPFLG_N_UNCOND_DIR_BR_CHK;
+            }
+            else if (strcmp(argv[optIdx], "-strict_br_cond") == 0)
+            {
+                add_create_flags |= OCSD_OPFLG_STRICT_N_UNCOND_BR_CHK;
+            }
+            else if (strcmp(argv[optIdx], "-range_cont") == 0)
+            {
+                add_create_flags |= OCSD_OPFLG_CHK_RANGE_CONTINUE;
+            }
+            else if (strcmp(argv[optIdx], "-halt_err") == 0)
+            {
+                add_create_flags |= OCSD_OPFLG_PKTDEC_HALT_BAD_PKTS;
+            }
+            else if (strcmp(argv[optIdx], "-aa64_opcode_chk") == 0)
+            {
+                add_create_flags |= ETM4_OPFLG_PKTDEC_AA64_OPCODE_CHK;
+            }
             else if (strcmp(argv[optIdx], "-macc_cache_disable") == 0)
             {
                 macc_cache_disable = true;
@@ -644,9 +679,11 @@ bool ProcessInputFile(DecodeTree *dcd_tree, std::string &in_filename,
                       TrcGenericElementPrinter* genElemPrinter, ocsdDefaultErrorLogger& err_logger)
 {
     bool bOK = true;
-
+    std::chrono::time_point<std::chrono::steady_clock> start, end;   // measure decode time
+    
     // need to push the data through the decode tree.
     std::ifstream in;
+
     in.open(in_filename, std::ifstream::in | std::ifstream::binary);
     if (in.is_open())
     {
@@ -655,6 +692,8 @@ bool ProcessInputFile(DecodeTree *dcd_tree, std::string &in_filename,
         uint8_t trace_buffer[bufferSize];   // temporary buffer to load blocks of data from the file
         uint32_t trace_index = 0;           // index into the overall trace buffer (file).
 
+        start = std::chrono::steady_clock::now();
+
         // process the file, a buffer load at a time
         while (!in.eof() && !OCSD_DATA_RESP_IS_FATAL(dataPathResp))
         {
@@ -745,7 +784,14 @@ bool ProcessInputFile(DecodeTree *dcd_tree, std::string &in_filename,
         in.close();
 
         std::ostringstream oss;
-        oss << "Trace Packet Lister : Trace buffer done, processed " << trace_index << " bytes.\n";
+        end = std::chrono::steady_clock::now();
+        std::chrono::duration<double> sec_elapsed{end -start};
+
+        oss << "Trace Packet Lister : Trace buffer done, processed " << trace_index << " bytes";
+        if (no_time_print)
+            oss << ".\n";
+        else
+            oss << " in " << std::setprecision(8) << sec_elapsed.count() << " seconds.\n";
         logger.LogMsg(oss.str());
         if (stats)
             PrintDecodeStats(dcd_tree);
@@ -768,10 +814,11 @@ bool ProcessInputFile(DecodeTree *dcd_tree, std::string &in_filename,
 void ListTracePackets(ocsdDefaultErrorLogger &err_logger, SnapShotReader &reader, const std::string &trace_buffer_name)
 {
     CreateDcdTreeFromSnapShot tree_creator;
+    uint32_t createFlags = add_create_flags;
 
     tree_creator.initialise(&reader, &err_logger);
 
-    if(tree_creator.createDecodeTree(trace_buffer_name, (decode == false), src_addr_n ? ETE_OPFLG_PKTDEC_SRCADDR_N_ATOMS : 0))
+    if(tree_creator.createDecodeTree(trace_buffer_name, (decode == false), createFlags))
     {
         DecodeTree *dcd_tree = tree_creator.getDecodeTree();
         dcd_tree->setAlternateErrorLogger(&err_logger);
@@ -801,10 +848,8 @@ void ListTracePackets(ocsdDefaultErrorLogger &err_logger, SnapShotReader &reader
             }
             if (macc_cache_disable || macc_cache_page_size || macc_cache_page_num)
             {
-                TrcMemAccMapper* pMapper = dcd_tree->getMemAccMapper();
-
                 if (macc_cache_disable)
-                    pMapper->enableCaching(false);
+                    dcd_tree->setMemAccCacheing(false, 0, 0);
                 else 
                 {
                     // one value set - set the other to default
@@ -812,7 +857,7 @@ void ListTracePackets(ocsdDefaultErrorLogger &err_logger, SnapShotReader &reader
                         macc_cache_page_size = MEM_ACC_CACHE_DEFAULT_PAGE_SIZE;
                     if (!macc_cache_page_num)
                         macc_cache_page_num = MEM_ACC_CACHE_DEFAULT_MRU_SIZE;
-                    pMapper->setCacheSizes(macc_cache_page_size, macc_cache_page_num);
+                    dcd_tree->setMemAccCacheing(true, macc_cache_page_size, macc_cache_page_num);
                 }
             }
         }
